#include <stdafx.h>
#include <locale.h>
#include "emule.h"
#include "OtherFunctions.h"

CString GetResString(UINT uStringID,WORD languageID) {
	CString resString;
	resString.LoadString(::GetModuleHandle(NULL),uStringID, languageID );
	if (!resString.GetLength())
		resString.LoadString(::GetModuleHandle(NULL),uStringID, MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT) );
	return resString;
}

CString GetResString(UINT uStringID) {
	return GetResString(uStringID,theApp.glob_prefs->GetLanguageID());
}

struct SLanguage {
	LANGID	lid;
	LPCTSTR pszLocale;
};

static const SLanguage _aLanguages[] =
{
	{MAKELANGID(LANG_ARABIC ,SUBLANG_ARABIC_UAE),				_T("")},
	{MAKELANGID(LANG_BULGARIAN,SUBLANG_DEFAULT),				_T("hun")},
	{MAKELANGID(LANG_CATALAN,SUBLANG_DEFAULT),					_T("")},
	{MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_SIMPLIFIED),		_T("chs")},
	{MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_TRADITIONAL),		_T("cht")},
	{MAKELANGID(LANG_DANISH,SUBLANG_DEFAULT),					_T("danish")},
	{MAKELANGID(LANG_DUTCH,SUBLANG_DEFAULT),					_T("dutch")},
	{MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT),					_T("english")},
	{MAKELANGID(LANG_ESTONIAN ,SUBLANG_DEFAULT),				_T("")},
	{MAKELANGID(LANG_FINNISH,SUBLANG_DEFAULT),					_T("finnish")},
	{MAKELANGID(LANG_FRENCH,SUBLANG_DEFAULT),					_T("french")},
	{MAKELANGID(LANG_GALICIAN ,SUBLANG_DEFAULT),				_T("")},
	{MAKELANGID(LANG_GERMAN,SUBLANG_DEFAULT),					_T("german")},
	{MAKELANGID(LANG_GREEK,SUBLANG_DEFAULT),					_T("greek")},
	{MAKELANGID(LANG_HUNGARIAN ,SUBLANG_DEFAULT),				_T("hungarian")},
	{MAKELANGID(LANG_ITALIAN,SUBLANG_DEFAULT),					_T("italian")},
	{MAKELANGID(LANG_KOREAN,SUBLANG_DEFAULT),					_T("korean")},
	{MAKELANGID(LANG_LATVIAN ,SUBLANG_DEFAULT),					_T("")},
	{MAKELANGID(LANG_LITHUANIAN,SUBLANG_DEFAULT),				_T("")},
	{MAKELANGID(LANG_NORWEGIAN,SUBLANG_NORWEGIAN_BOKMAL),		_T("norwegian")},
	{MAKELANGID(LANG_POLISH,SUBLANG_DEFAULT),					_T("polish")},
	{MAKELANGID(LANG_PORTUGUESE,SUBLANG_PORTUGUESE),			_T("ptg")},
	{MAKELANGID(LANG_PORTUGUESE,SUBLANG_PORTUGUESE_BRAZILIAN),	_T("ptb")},
	{MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT),					_T("russian")},
	{MAKELANGID(LANG_SLOVENIAN,SUBLANG_DEFAULT),				_T("")},
	{MAKELANGID(LANG_SPANISH,SUBLANG_SPANISH),					_T("spanish")},
	{MAKELANGID(LANG_SWEDISH,SUBLANG_DEFAULT),					_T("swedish")},
	{MAKELANGID(LANG_TURKISH ,SUBLANG_DEFAULT),					_T("turkish")},
	{0, NULL}
};

void CPreferences::GetLanguages(CWordArray& aLanguageIDs) const
{
	const SLanguage* pLang = _aLanguages;
	while (pLang->lid)
		aLanguageIDs.Add((pLang++)->lid);
}

WORD CPreferences::GetLanguageID() const
{
	return prefs->languageID;
}

void CPreferences::SetLanguageID(WORD lid)
{
	prefs->languageID = lid;
}

void CPreferences::SetLanguage()
{
	bool bFoundLang = false;
	if (prefs->languageID){
		const SLanguage* pLangs = _aLanguages;
		if (pLangs){
			while (pLangs->lid){
				if (pLangs->lid == prefs->languageID){
					setlocale(LC_ALL, pLangs->pszLocale);
					bFoundLang = true;
					break;
				}
				pLangs++;
			}
		}
	}

	if (!bFoundLang){
		// no preset language available - get system language and check if its supported (cases).
		uint8 nLocalLang = ::GetThreadLocale();
		uint8 langp=PRIMARYLANGID(::GetThreadLocale());
		uint8 langs=SUBLANGID(::GetThreadLocale());

        CString strLng1 = GetResString(IDS_MB_LANGUAGEINFO,MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
		CString strLng2 = "";
		switch(nLocalLang) {
			case	LANG_ARABIC:
			case	LANG_BULGARIAN:
			case	LANG_CATALAN:
			case	LANG_ESTONIAN:
			case	LANG_FINNISH:
			case	LANG_GALICIAN:
			case	LANG_GREEK:
			case	LANG_HUNGARIAN:
			case	LANG_ITALIAN:
			case	LANG_LATVIAN:
			case	LANG_TURKISH:
			case 	LANG_CHINESE:
			case 	LANG_DANISH:
			case 	LANG_DUTCH:
			case 	LANG_FRENCH:
			case 	LANG_GERMAN:
			case 	LANG_KOREAN:
			case 	LANG_LITHUANIAN:
			case 	LANG_NORWEGIAN:
			case 	LANG_POLISH:
			case 	LANG_PORTUGUESE:
			case 	LANG_RUSSIAN:
			case 	LANG_SLOVENIAN:
			case 	LANG_SPANISH:
			case 	LANG_SWEDISH:
			{
				prefs->languageID =  MAKELANGID(langp,langs);
				SetThreadLocale(prefs->languageID);
				if (nLocalLang != LANG_ENGLISH) strLng2=GetResString(IDS_MB_LANGUAGEINFO,prefs->languageID);
				break;
			}
			default:	// not supported language -> use english
				prefs->languageID = MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT);
				SetThreadLocale(prefs->languageID);
		}

		CString strMessage = strLng2 + CString("\n\n") + strLng1;
		AfxMessageBox(strMessage,MB_ICONASTERISK);
	}
	SetThreadLocale(prefs->languageID);

	// if loading a string fails, set language to English
	if (GetResString(IDS_MB_LANGUAGEINFO,prefs->languageID).GetLength()==0) {
		prefs->languageID = MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT);
		SetThreadLocale(prefs->languageID);
	}
}
